if (Nanika:GetClass() ~= "PALADIN") then return end

-- Local initialisations
Nanika.Paladin = Nanika:NewModule("Paladin")
Nanika.Paladin.Blessings = Nanika.Paladin:NewModule("Blessings", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0")
local Paladin = Nanika.Paladin
local Blessings = Nanika.Paladin.Blessings
local settings = {}
local defaults = {}

-- Ace functions
function Paladin:OnInitialize()
	self.db = Nanika.db:RegisterNamespace("Paladin")
end

function Blessings:OnInitialize() -- Called straight after UI load
	-- slash commands
	self:RegisterChatCommand("nanikablessings", "OnSlashCommand")
	-- default settings
	defaults.CurrentSpellIndex = {}
	defaults.CurrentTargets = {}
	-- set up db
	local dbsettings = { profile = defaults }
	self.db = Nanika.db:RegisterNamespace("Paladin Blessings")
	self.db:RegisterDefaults(dbsettings)
	settings = self.db.profile

	self.RegisterEvents = {"PLAYER_REGEN_ENABLED","PLAYER_SPECIALIZATION_CHANGED","PET_BATTLE_OPENING_START","PET_BATTLE_CLOSE"}
	self.Count = 3
	self.UI = {}
	self.UI.BaseName = "NanikaPaladinBlessings"
	self.UI.Frame = NanikaPaladinBlessingsFrame
	self.Spells = {
		[0] = GetSpellInfo(203528), --GreaterBlessingOfKings
		[1] = GetSpellInfo(203538), --GreaterBlessingOfKings
		[2] = GetSpellInfo(203539), --GreaterBlessingOfWisdom
	}
	self.SpellCount = 3
	self.UI.Buttons = {}
end
function Blessings:OnSlashCommand(input)
	if input == "reset" then
		self.db:ResetProfile()
		ReloadUI()
	return end
end
function Blessings:OnEnable() -- Called after OnInitialize if enabled and every time when switching from disabled to enabled
	for i, value in ipairs(self.RegisterEvents) do self:RegisterEvent(value) end
	for i = 0, self.Count - 1 do self:CreateBlessingButton(i) end
	self:Show()
end
function Blessings:OnDisable() -- Called when switching from enabled to disabled
	for i, value in ipairs(self.RegisterEvents) do self:UnregisterEvent(value) end	
end

-- Show & Hide
function Blessings:Show()
	if (GetSpecialization() ~= 3) then self:Hide() return end
	if (self.UI.Frame:IsShown()) then return end
	self.UI.Frame:Show()
	for i = 0, self.Count - 1 do
		local button = self.UI.Buttons[i]
		button:Show()
		button:ClearAllPoints()
		button:SetPoint("TOP", self.UI.Frame, "BOTTOM", 0, -(button:GetHeight() * i))
	end
	self.UI.RefreshTimer = self:ScheduleRepeatingTimer("OnRefresh", 1)
end
function Blessings:Hide()
	self.UI.Frame:Hide()
	if (self.UI.RefreshTimer ~= nil) then self:CancelTimer(self.UI.RefreshTimer) return end
end

-- UI
function Blessings:CreateBlessingButton(i)
	local name = self.UI.BaseName.."Button"..i
	local basename = "SecureActionButtonTemplate, "..self.UI.BaseName.."ButtonBase"
	local button = CreateFrame("Button", name, self.UI.Frame, basename)
	self.UI.Buttons[i] = button
	button.Index = i
	button:SetWidth(self.UI.Frame:GetWidth())
	button.IconFrame = _G[name.."IconFrame"]
	button.IconTexture = _G[name.."IconFrameIconTexture"]
	button.TargetString = _G[name.."TargetString"]
	button.ExpirationString = _G[name.."ExpirationString"]
	button:RegisterForClicks("LeftButtonDown", "RightButtonDown")

	-- Left mouse button down (cast blessing)
	button:SetAttribute("type1", "spell")

	-- Right mouse button down (show target selection menu)
	button:SetAttribute("type2", "targetmenu")
	button.targetmenu = function() 
        local menu = CreateFrame("Frame", name.."DropDownMenu", UIParent, "UIDropDownMenuTemplate");
        menu.initialize = function(dropdown, level, menuList) self:InitializeTargetSelectionMenu(button, dropdown, level, menuList) end
        menu.displayMode = "MENU"
        ToggleDropDownMenu(1, nil, menu, "cursor", 10, -8)
	end

	-- Mouse wheel up/down (change blessing)
	button:EnableMouseWheel(true)
	button:SetScript("OnMouseWheel", function(iconframe, delta)
		self:AssignSpell(button, button.CurrentSpellIndex + delta)
	end)
	self:AssignSpell(button, settings.CurrentSpellIndex[i] or (i + 1))
	self:AssignTarget(button, UnitName(settings.CurrentTargets[i] or "player"), settings.CurrentTargets[i] or "player")
end
function Blessings:InitializeTargetSelectionMenu(button, dropdown, level, menuList)
	local info
	if level == 1 then
		info = UIDropDownMenu_CreateInfo()
		info.isTitle = 1
		info.text = "Select target"
		UIDropDownMenu_AddButton(info, 1)

		pname = UnitName("player")
		self:AddTargetSelectionOption(button, pname, pname, "player")

		tname, trealm = UnitName("target")
		if tname then
			if trealm and trealm ~= "" then tname = tname.."-"..trealm end
			self:AddTargetSelectionOption(button, "Target: "..tname, tname)
		end

		fname, frealm = UnitName("focus")
		if fname then
			if frealm and frealm ~= "" then fname = fname.."-"..frealm end
			self:AddTargetSelectionOption(button, "Focus: "..fname, fname)
		end

		raid = Nanika:GetRaidLayout()
		if raid then
			for i = 1, MAX_RAID_GROUPS do
				if raid[i].MemberCount > 0 then
					info = UIDropDownMenu_CreateInfo()
					info.text = "Party "..i
					info.notCheckable = true
					info.hasArrow = true
					info.menuList = raid[i]
					UIDropDownMenu_AddButton(info, 1)
				end
			end
		end
	elseif menuList then
		for i = 1, menuList.MemberCount do
			local member = menuList.Members[i]
			--if we assign a party/raid id then we risk the chance of it not being accurate when casting
			self:AddTargetSelectionOption(button, member.Name, member.Name, member.Name, 2)
		end
	end
end
function Blessings:AddTargetSelectionOption(button, labelname, targetname, targetunitid, level)
	targetname = targetname or labelname
	targetunitid = targetunitid or targetname
	info = UIDropDownMenu_CreateInfo()
	info.button = button
	info.text = labelname
	info.notCheckable = true
	if UnitIsPlayer(targetunitid) then
		info.func = function() self:AssignTarget(button, targetname, targetunitid) end
	else
		info.disabled = true
		info.tooltipTitle = "Title"
		info.tooltipText = "Tooltip"
	end
	UIDropDownMenu_AddButton(info, level or 1)
end

function Blessings:AssignSpell(button, i)
	i = (i + self.SpellCount) % self.SpellCount
	settings.CurrentSpellIndex[button.Index] = i
	button.CurrentSpellIndex = i
	local name,_,texture = GetSpellInfo(self.Spells[i])
	button:SetAttribute("spell1", name)
	button.IconTexture:SetTexture(texture)
	self:RefreshButton(button)
end
function Blessings:AssignTarget(button, name, unitid)
	settings.CurrentTargets[button.Index] = unitid
	button.TargetString:SetText(name)
	button:SetAttribute("unit1", unitid)
end

function Blessings:OnRefresh()
	for i = 0, self.Count - 1 do self:RefreshButton(self.UI.Buttons[i]) end
end
function Blessings:RefreshButton(button)
	local unitid = button:GetAttribute("unit1")
	local spellName = self.Spells[button.CurrentSpellIndex]
	if unitid and spellName then
		local _,_,_,_,_,buffDuration,buffExpiration = UnitBuff(unitid, spellName)
		if buffExpiration then
			local expires = max(0, buffExpiration - GetTime())
			local mins = floor(expires / 60)
			local secs = floor(expires % 60)
			local part = expires / buffDuration
			button.ExpirationString:SetText(string.format("%d:%02d", mins, secs))
			if part > 0.5 then
				button:SetBackdropColor((1 - part) * 2, 1, 0)
			else
				button:SetBackdropColor(1, part * 2, 0)
			end
		else
			button.ExpirationString:SetText("-")
			button:SetBackdropColor(1, 0, 0)
		end
	end
end
-- Events
function Blessings:PLAYER_REGEN_ENABLED() self:Show() end
function Blessings:PLAYER_SPECIALIZATION_CHANGED() self:Show() end
function Blessings:PET_BATTLE_OPENING_START() self:Hide() end
function Blessings:PET_BATTLE_CLOSE() self:Show() end
